# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2018-09-12 13:26
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('adyengo', '0020_auto_20170616_1329'),
    ]

    operations = [
        migrations.AlterField(
            model_name='notification',
            name='payment_amount',
            field=models.PositiveIntegerField(),
        ),
        migrations.AlterField(
            model_name='session',
            name='payment_amount',
            field=models.PositiveIntegerField(),
        ),
    ]



from django.db import models
from django.utils import timezone

from .managers import RecurringContractManager
from .utils import remove_items_with_empty_values, merchant_sig
from . import settings, constants, api


def tomorrow():
    return (timezone.now() + timezone.timedelta(days=1))


class Session(models.Model):

    session_type = models.CharField(max_length=25, choices=sorted(constants.SESSION_TYPES.items()))
    merchant_reference = models.CharField(max_length=80, unique=True)
    payment_amount = models.PositiveIntegerField()
    currency_code = models.CharField(
        max_length=3,
        choices=sorted(constants.CURRENCY_CODES.items()),
        default=settings.DEFAULT_CURRENCY_CODE
    )
    ship_before_date = models.DateTimeField(default=tomorrow, null=True)
    skin_code = models.CharField(max_length=10, default=settings.DEFAULT_SKIN_CODE)
    shopper_locale = models.CharField(
        max_length=5,
        choices=sorted(constants.LOCALES.items()),
        default=settings.DEFAULT_SHOPPER_LOCALE,
        blank=True
    )
    order_data = models.TextField(blank=True)
    session_validity = models.DateTimeField(default=tomorrow, null=True)
    merchant_return_data = models.CharField(max_length=128)
    country_code = models.CharField(max_length=2, choices=sorted(constants.COUNTRY_CODES.items()))
    shopper_email = models.EmailField(blank=True)
    shopper_reference = models.CharField(max_length=80, blank=True)
    shopper_ip = models.CharField(max_length=45, blank=True)
    shopper_statement = models.CharField(max_length=135, blank=True)
    fraud_offset = models.PositiveIntegerField(null=True)
    recurring_contract = models.CharField(
        max_length=50,
        choices=sorted(constants.RECURRING_CONTRACT_TYPES_PLUS_COMBOS.items()),
        blank=True
    )
    recurring_detail_reference = models.CharField(max_length=80, blank=True)
    res_url = models.CharField(max_length=2000, default=settings.DEFAULT_RES_URL, null=True, blank=True)
    page_type = models.CharField(
        max_length=15,
        choices=sorted(constants.PAGE_TYPES.items()),
        default=settings.DEFAULT_PAGE_TYPE
    )
    creation_time = models.DateTimeField(auto_now_add=True)

    def __str__(self):
        return self.merchant_reference

    class Meta:
        ordering = ('-creation_time',)

    def is_hpp_regular(self):
        return self.session_type == constants.SESSION_TYPE_HPP_REGULAR

    def is_hpp_recurring(self):
        return self.session_type == constants.SESSION_TYPE_HPP_RECURRING

    def is_api_recurring(self):
        return self.session_type == constants.SESSION_TYPE_API_RECURRING

    def url(self):

        if self.page_type == constants.PAGE_TYPE_MULTIPLE:
            return settings.PAYMENT_PAGES_MULTIPLE_URL

        if self.page_type == constants.PAGE_TYPE_SINGLE:
            return settings.PAYMENT_PAGES_SINGLE_URL

        if self.page_type == constants.PAGE_TYPE_SKIP:
            return settings.PAYMENT_PAGES_SKIP_URL

    def hpp_params(self):

        params = {
            'merchantReference': self.merchant_reference,
            'paymentAmount': self.payment_amount,
            'currencyCode': self.currency_code,
            'shipBeforeDate': self.ship_before_date.isoformat(),
            'skinCode': self.skin_code,
            'merchantAccount': settings.MERCHANT_ACCOUNT,
            'shopperLocale': self.shopper_locale,
            'orderData': self.order_data,
            'sessionValidity': self.session_validity.isoformat(),
            'merchantReturnData': self.merchant_return_data,
            'countryCode': self.country_code,
            'shopperEmail': self.shopper_email,
            'shopperReference': self.shopper_reference,
            'recurringContract': self.recurring_contract,
            'selectedRecurringDetailReference': self.recurring_detail_reference,
            'resURL': self.res_url,
            'allowedMethods': ','.join([
                m.method for m in self.allowed_payment_methods.all()
            ]),
            'blockedMethods': ','.join([
                m.method for m in self.blocked_payment_methods.all()
            ])
        }

        if self.page_type == constants.PAGE_TYPE_SKIP:
            params['brandCode'] = params['allowedMethods']

        params = remove_items_with_empty_values(params)
        params['merchantSig'] = merchant_sig(params)

        return params

    def exec_recurring_payment(self):
        """
        Executes the API call to Adyen to submit a recurring payment.

        Returns the result Adyen sends back as a dict with these keys:

        psp_reference       - The reference Adyen assigned to the payment. This
                              is guaranteed to be globally unique and must be
                              used when communicating about this payment with
                              Adyen.
        result_code         - The result of the payment. One of Authorised,
                              Refused or Error.
        auth_code           - An authorisation code if the payment was
                              successful, or blank otherwise.
        refusal_reason      - If the payment was refused, the refusal reason.
        """

        if self.recurring_contract == constants.RECURRING_CONTRACT_TYPE_RECURRING:
            shopper_interaction = 'ContAuth'
        elif self.recurring_contract == constants.RECURRING_CONTRACT_TYPE_ONECLICK:
            shopper_interaction = 'Ecommerce'

        # if self.fraud_offset:
        #     request['fraudOffset'] = self.fraud_offset

        # if self.shopper_statement:
        #     request['shopperStatement'] = self.shopper_statement

        # if self.shopper_ip:
        #     request['shopperIp'] = self.shopper_ip

        response = api.exec_recurring_payment(
            contract_type=self.recurring_contract,
            shopper_interaction=shopper_interaction,
            shopper_reference=self.shopper_reference,
            shopper_email=self.shopper_email,
            merchant_reference=self.merchant_reference,
            payment_amount=self.payment_amount,
            currency_code=self.currency_code
        )

        r = RecurringPaymentResult(
            session=self,
            psp_reference=response.get('pspReference')
        )

        if response.get('errorCode'):
            r.result_code = constants.RECURRING_PAYMENT_RESULT_ERROR
            r.refusal_reason = response.get('message')
        else:
            r.result_code = response.get('resultCode')
            r.auth_code = response.get('authCode')
            r.refusal_reason = response.get('refusalReason', '')

        r.save()

        return r

    def flush_recurring_contract_cache(self):
        """
        Flushes the Recurring Payment Contract cache we have for this
        shopper reference and contract type.

        This methods should be called when a session is authorized that
        potentially results in a new Recurring Payment Contract.

        We shouldn't try to refill the cache here right away, because sometimes
        there's a delay in the creation of the contracts at Adyen. Therefor
        it's best to always fetch the contracts from Adyen just before the
        payment.
        """
        # Split by comma because it can be a combination like
        # `ONECLICK,RECURRING`. In that case we want both contract types to be
        # flushed (because the potential new contract can be made for either).
        for c in self.recurring_contract.split(','):
            RecurringContract.objects.flush_cache(self.shopper_reference, c)


class SessionAllowedPaymentMethods(models.Model):

    session = models.ForeignKey(Session, on_delete=models.CASCADE, related_name='allowed_payment_methods')
    method = models.CharField(max_length=50, choices=sorted(constants.PAYMENT_METHODS.items()))

    def __str__(self):
        return self.method


class SessionBlockedPaymentMethods(models.Model):

    session = models.ForeignKey(Session, on_delete=models.CASCADE, related_name='blocked_payment_methods')
    method = models.CharField(max_length=50, choices=sorted(constants.PAYMENT_METHODS.items()))

    def __str__(self):
        return self.method


class RecurringContract(models.Model):

    recurring_detail_reference = models.CharField(max_length=150, blank=True)
    shopper_reference = models.CharField(max_length=80)
    contract_type = models.CharField(max_length=50)
    payment_method_type = models.CharField(max_length=15)
    variant = models.CharField(max_length=50, blank=True)
    creation_date = models.DateTimeField(blank=True)
    creation_time = models.DateTimeField(auto_now_add=True)

    objects = RecurringContractManager()

    def __str__(self):
        return self.recurring_detail_reference

    class Meta:
        ordering = ('-creation_time',)
        unique_together = ('recurring_detail_reference', 'shopper_reference')

    @property
    def details_dict(self):
        """
        Returns a dict of the contract's details.
        """
        return {detail.key: detail.value for detail in self.details.all()}


class RecurringContractDetail(models.Model):

    recurring_contract = models.ForeignKey(RecurringContract, related_name='details')
    key = models.CharField(max_length=100)
    value = models.CharField(max_length=250, blank=True)

    def __str__(self):
        return self.key


class RecurringPaymentResult(models.Model):

    session = models.ForeignKey(Session, on_delete=models.CASCADE, related_name='recurring_payment_results')
    psp_reference = models.BigIntegerField()
    result_code = models.CharField(max_length=30, choices=sorted(constants.RECURRING_PAYMENT_RESULT_CODES.items()))
    auth_code = models.PositiveIntegerField(null=True)
    refusal_reason = models.CharField(max_length=250, blank=True)

    def is_authorized(self):
        return self.result_code == constants.RECURRING_PAYMENT_RESULT_AUTHORISED

    def is_received(self):
        return self.result_code == constants.RECURRING_PAYMENT_RESULT_RECEIVED

    def is_refused(self):
        return self.result_code == constants.RECURRING_PAYMENT_RESULT_REFUSED

    def is_error(self):
        return self.result_code == constants.RECURRING_PAYMENT_RESULT_ERROR


class Notification(models.Model):

    ip_address = models.CharField(max_length=45, null=True, blank=True)
    live = models.BooleanField()
    event_code = models.CharField(max_length=50, null=True, blank=True)
    psp_reference = models.CharField(max_length=150, null=True, blank=True)
    original_reference = models.CharField(max_length=150, null=True, blank=True)
    merchant_reference = models.CharField(max_length=128, null=True, blank=True)
    merchant_account_code = models.CharField(max_length=150, null=True, blank=True)
    event_date = models.DateTimeField(null=True, blank=True)
    success = models.BooleanField()
    payment_method = models.CharField(
        max_length=50,
        choices=sorted(constants.PAYMENT_METHODS.items()),
        null=True,
        blank=True
    )
    operations = models.CharField(max_length=100, null=True, blank=True)
    reason = models.CharField(max_length=250, null=True, blank=True)
    payment_amount = models.PositiveIntegerField()
    currency_code = models.CharField(
        max_length=3,
        choices=sorted(constants.CURRENCY_CODES.items()),
        default=settings.DEFAULT_CURRENCY_CODE
    )
    session = models.ForeignKey(Session, on_delete=models.CASCADE, null=True, blank=True)
    creation_time = models.DateTimeField(auto_now_add=True)

    def __str__(self):
        return '{} {}'.format(self.event_code, self.psp_reference)

    def save(self, *args, **kwargs):

        # Try to find the session that initiated this notification. Generally,
        # this should always exist, but we don't want the notification to fail
        # in case it doesn't, therefor the `try` statement.
        try:
            self.session = Session.objects.get(
                merchant_reference=self.merchant_reference
            )
        except Exception:
            pass

        super(Notification, self).save(*args, **kwargs)

        # If the session that this notification initiated was a HPP recurring
        # session, then the Recurring Contracts cache should be flushed.
        if self.session and self.session.is_hpp_recurring():
            self.session.flush_recurring_contract_cache()

    class Meta:
        ordering = ('-creation_time',)
        unique_together = ('live', 'merchant_account_code', 'psp_reference', 'event_code', 'success')

    def is_status(self, status_code):
        return (
            self.success and
            self.event_code == status_code
        )

    def is_authorized(self):
        return self.is_status(constants.NOTIFICATION_EVENT_CODE_AUTHORISATION)

    def is_cancelled(self):
        return self.is_status(constants.NOTIFICATION_EVENT_CODE_CANCELLATION)

    def is_refunded(self):
        return self.is_status(constants.NOTIFICATION_EVENT_CODE_REFUND)

    def is_canceled_or_refunded(self):
        return self.is_status(constants.NOTIFICATION_EVENT_CODE_CANCEL_OR_REFUND)

    def is_captured(self):
        return self.is_status(constants.NOTIFICATION_EVENT_CODE_CAPTURE)

    def is_refuned_reversed(self):
        return self.is_status(constants.NOTIFICATION_EVENT_CODE_REFUNDED_REVERSED)

    def is_capture_failed(self):
        return self.is_status(constants.NOTIFICATION_EVENT_CODE_CAPTURE_FAILED)

    def is_refund_failed(self):
        return self.is_status(constants.NOTIFICATION_EVENT_CODE_REFUND_FAILED)

    def is_request_for_information(self):
        return self.is_status(constants.NOTIFICATION_EVENT_CODE_REQUEST_FOR_INFORMATION)

    def is_notification_of_chargeback(self):
        return self.is_status(constants.NOTIFICATION_EVENT_CODE_NOTIFICATION_OF_CHARGEBACK)

    def is_advice_of_debit(self):
        return self.is_status(constants.NOTIFICATION_EVENT_CODE_ADVICE_OF_DEBIT)

    def is_chargedback(self):
        return self.is_status(constants.NOTIFICATION_EVENT_CODE_CHARGEBACK)

    def is_chargeback_reversed(self):
        return self.is_status(constants.NOTIFICATION_EVENT_CODE_CHARGEBACK_REVERSED)

    def is_report_available(self):
        return self.is_status(constants.NOTIFICATION_EVENT_CODE_REPORT_AVAILABLE)



# Session Types

SESSION_TYPE_HPP_REGULAR = 'hpp_regular'
SESSION_TYPE_HPP_RECURRING = 'hpp_recurring'
SESSION_TYPE_API_RECURRING = 'api_recurring'

SESSION_TYPES_HPP = {
    SESSION_TYPE_HPP_REGULAR: "HPP Regular",
    SESSION_TYPE_HPP_RECURRING: "HPP Recurring",
}

SESSION_TYPES_API = {
    SESSION_TYPE_API_RECURRING: "API Recurring"
}

SESSION_TYPES = SESSION_TYPES_HPP.copy()
SESSION_TYPES.update(SESSION_TYPES_API)


# Page Types

PAGE_TYPE_MULTIPLE = 'multiple'
PAGE_TYPE_SINGLE = 'single'
PAGE_TYPE_SKIP = 'skip'

PAGE_TYPES = {
    PAGE_TYPE_MULTIPLE: "Multiple",
    PAGE_TYPE_SINGLE: "Single",
    PAGE_TYPE_SKIP: "Skip"
}


# Currency Code

CURRENCY_CODE_EUR = 'EUR'

CURRENCY_CODES = {
    CURRENCY_CODE_EUR: "Euro",
}


# Locales

LOCALE_DE_DE = 'de_DE'
LOCALE_EN_GB = 'en_GB'
LOCALE_FR_BE = 'fr_BE'
LOCALE_NL_BE = 'nl_BE'
LOCALE_NL_NL = 'nl_NL'

LOCALES = {
    LOCALE_DE_DE: "German (Germany)",
    LOCALE_EN_GB: "English (United Kingdom)",
    LOCALE_FR_BE: "French (Belgium)",
    LOCALE_NL_BE: "Dutch (Belgium)",
    LOCALE_NL_NL: "Dutch (Holland)"
}

# Country Code

COUNTRY_CODE_BE = 'BE'
COUNTRY_CODE_DE = 'DE'
COUNTRY_CODE_GB = 'GB'
COUNTRY_CODE_NL = 'NL'

COUNTRY_CODES = {
    COUNTRY_CODE_BE: "Belgium",
    COUNTRY_CODE_DE: "Germany",
    COUNTRY_CODE_GB: "United Kingdom",
    COUNTRY_CODE_NL: "Netherlands"
}

# Payment Method

PAYMENT_METHOD_AMEX = 'amex'
PAYMENT_METHOD_BANKTRANSFER = 'bankTransfer'
PAYMENT_METHOD_BANKTRANSFER_DE = 'bankTransfer_DE'
PAYMENT_METHOD_BANKTRANSFER_IBAN = 'bankTransfer_IBAN'
PAYMENT_METHOD_BANKTRANSFER_NL = 'bankTransfer_NL'
PAYMENT_METHOD_BCMC = 'bcmc'
PAYMENT_METHOD_CARD = 'card'
PAYMENT_METHOD_DINERS = 'diners'
PAYMENT_METHOD_DIRECTDEBIT_NL = 'directdebit_NL'
PAYMENT_METHOD_DIRECT_E_BANKING = 'directEbanking'
PAYMENT_METHOD_DISCOVER = 'discover'
PAYMENT_METHOD_DOTPAY = 'dotpay'
PAYMENT_METHOD_EBANKING_FI = 'ebanking_FI'
PAYMENT_METHOD_ELV = 'elv'
PAYMENT_METHOD_GIROPAY = 'giropay'
PAYMENT_METHOD_IDEAL = 'ideal'
PAYMENT_METHOD_MAESTRO = 'maestro'
PAYMENT_METHOD_MC = 'mc'
PAYMENT_METHOD_PAYPAL = 'paypal'
PAYMENT_METHOD_SEPADIRECTDEBIT = 'sepadirectdebit'
PAYMENT_METHOD_VISA = 'visa'

PAYMENT_METHODS = {
    PAYMENT_METHOD_AMEX: "Amex",
    PAYMENT_METHOD_BANKTRANSFER: "All banktransfers",
    PAYMENT_METHOD_BANKTRANSFER_DE: "German Banktransfer",
    PAYMENT_METHOD_BANKTRANSFER_IBAN: "International Bank Transfer (IBAN)",
    PAYMENT_METHOD_BANKTRANSFER_NL: "Dutch Banktransfer",
    PAYMENT_METHOD_BCMC: "Bancontact card",
    PAYMENT_METHOD_CARD: "All debit and credit cards",
    PAYMENT_METHOD_DINERS: "Diners Club",
    PAYMENT_METHOD_DIRECTDEBIT_NL: "Direct Debit (Netherlands)",
    PAYMENT_METHOD_DIRECT_E_BANKING: "SofortUberweisung",
    PAYMENT_METHOD_DISCOVER: "Discover",
    PAYMENT_METHOD_DOTPAY: "Dotpay",
    PAYMENT_METHOD_EBANKING_FI: "Finnish E-Banking",
    PAYMENT_METHOD_ELV: "ELV",
    PAYMENT_METHOD_GIROPAY: "GiroPay",
    PAYMENT_METHOD_IDEAL: "iDEAL",
    PAYMENT_METHOD_MAESTRO: "Maestro",
    PAYMENT_METHOD_MC: "Master Card",
    PAYMENT_METHOD_PAYPAL: "PayPal",
    PAYMENT_METHOD_SEPADIRECTDEBIT: "SEPA Direct Debit",
    PAYMENT_METHOD_VISA: "Visa",
}


# Recurring contract types

RECURRING_CONTRACT_TYPE_RECURRING = 'RECURRING'
RECURRING_CONTRACT_TYPE_ONECLICK = 'ONECLICK'

RECURRING_CONTRACT_TYPES = {
    RECURRING_CONTRACT_TYPE_RECURRING: "Recurring",
    RECURRING_CONTRACT_TYPE_ONECLICK: "One click"
}

# Recurring contract types plus combinations

RECURRING_CONTRACT_TYPES_PLUS_COMBOS = RECURRING_CONTRACT_TYPES.copy()
RECURRING_CONTRACT_TYPES_PLUS_COMBOS.update({
    '{},{}'.format(
        RECURRING_CONTRACT_TYPE_RECURRING,
        RECURRING_CONTRACT_TYPE_ONECLICK
    ): "Recurring and One click (user chooses)"
})


# Recurring contract variant fields

RECURRING_CONTRACT_VARIANT_FIELDS = {
    'card': (
        'expiryMonth',
        'expiryYear',
        'holderName',
        'number',
        'cvc',
        'issueNumber',
        'startMonth',
        'startYear'
    ),
    'elv': (
        'bankLocation',
        'bankName',
        'bankLocationId',
        'accountHolderName',
        'bankAccountNumber'
    ),
    'bank': (
        'bankAccountNumber',
        'bankLocationId',
        'bankName',
        'bic',
        'countryCode',
        'iban',
        'ownerName'
    )
}


# Recurring payment result codes

RECURRING_PAYMENT_RESULT_AUTHORISED = 'Authorised'
RECURRING_PAYMENT_RESULT_REFUSED = 'Refused'
RECURRING_PAYMENT_RESULT_RECEIVED = 'Received'
RECURRING_PAYMENT_RESULT_ERROR = 'Error'

RECURRING_PAYMENT_RESULT_CODES = {
    RECURRING_PAYMENT_RESULT_AUTHORISED: 'Authorised',
    RECURRING_PAYMENT_RESULT_REFUSED: 'Refused',
    RECURRING_PAYMENT_RESULT_RECEIVED: 'Received',
    RECURRING_PAYMENT_RESULT_ERROR: 'Error'
}


# Notification event codes

NOTIFICATION_EVENT_CODE_AUTHORISATION = 'AUTHORISATION'
NOTIFICATION_EVENT_CODE_CANCELLATION = 'CANCELLATION'
NOTIFICATION_EVENT_CODE_REFUND = 'REFUND'
NOTIFICATION_EVENT_CODE_CANCEL_OR_REFUND = 'CANCEL_OR_REFUND'
NOTIFICATION_EVENT_CODE_CAPTURE = 'CAPTURE'
NOTIFICATION_EVENT_CODE_REFUNDED_REVERSED = 'REFUNDED_REVERSED'
NOTIFICATION_EVENT_CODE_CAPTURE_FAILED = 'CAPTURE_FAILED'
NOTIFICATION_EVENT_CODE_REFUND_FAILED = 'REFUND_FAILED'
NOTIFICATION_EVENT_CODE_REQUEST_FOR_INFORMATION = 'REQUEST_FOR_INFORMATION'
NOTIFICATION_EVENT_CODE_NOTIFICATION_OF_CHARGEBACK = 'NOTIFICATION_OF_CHARGEBACK'
NOTIFICATION_EVENT_CODE_ADVICE_OF_DEBIT = 'ADVICE_OF_DEBIT'
NOTIFICATION_EVENT_CODE_CHARGEBACK = 'CHARGEBACK'
NOTIFICATION_EVENT_CODE_CHARGEBACK_REVERSED = 'CHARGEBACK_REVERSED'
NOTIFICATION_EVENT_CODE_REPORT_AVAILABLE = 'REPORT_AVAILABLE'

NOTIFICATION_EVENT_CODES = {
    NOTIFICATION_EVENT_CODE_AUTHORISATION: "Authorisation",
    NOTIFICATION_EVENT_CODE_CANCELLATION: "Cancellation",
    NOTIFICATION_EVENT_CODE_REFUND: "Refund",
    NOTIFICATION_EVENT_CODE_CANCEL_OR_REFUND: "Cancel or refund",
    NOTIFICATION_EVENT_CODE_CAPTURE: "Capture",
    NOTIFICATION_EVENT_CODE_REFUNDED_REVERSED: "Refunded reversed",
    NOTIFICATION_EVENT_CODE_CAPTURE_FAILED: "Capture failed",
    NOTIFICATION_EVENT_CODE_REFUND_FAILED: "Refund failed",
    NOTIFICATION_EVENT_CODE_REQUEST_FOR_INFORMATION: "Request for information",
    NOTIFICATION_EVENT_CODE_NOTIFICATION_OF_CHARGEBACK: "Notification of chargeback",
    NOTIFICATION_EVENT_CODE_ADVICE_OF_DEBIT: "Advice of debit",
    NOTIFICATION_EVENT_CODE_CHARGEBACK: "Chargeback",
    NOTIFICATION_EVENT_CODE_CHARGEBACK_REVERSED: "Chargeback reversed",
    NOTIFICATION_EVENT_CODE_REPORT_AVAILABLE: "Report available"
}


# Adyen servers ip addresses

ADYEN_SERVERS_IP_ADDRESS_RANGES = (
    u'82.199.87.128/26',  # 82.199.87.129 to 82.199.87.191
    u'82.199.90.136/29',  # 82.199.90.137 to 82.199.90.142
    u'82.199.90.160/27',  # 82.199.90.161 to 82.199.90.190
    u'91.212.42.0/24'     # 91.212.42.1 to 91.212.42.254
)

# Only these IP addresses should be allowed to send notifications.
# You can check if an IP exists inside a range using ipaddress:
# https://pypi.python.org/pypi/ipaddress/1.0.4
#
# import ipaddress
# ipaddress.ip_address(u'82.199.87.135') in ipaddress.ip_network(u'82.199.87.128/26')
#
# Note that both IP addresses SHOULD be unicode.



from django.conf import settings
from django.conf.urls import url
from django.views.generic.base import RedirectView
from . import views


class AdyengoSite(object):

    @property
    def urls(self):

        if not settings.DEBUG:
            raise Exception(
                "Please don't include Adyengo test page URL's when "
                "`DEBUG` is `False`."
            )

        urlpatterns = [
            url(r'^$', RedirectView.as_view(url='hpp/setup_session/')),
            url(r'^hpp/setup_session/((?P<session_type>\w+)/)?$', views.hpp_setup_session, name='hpp_setup_session'),
            url(r'^hpp/dispatch_session/$', views.hpp_dispatch_session, name='hpp_dispatch_session'),
            url(r'^hpp/result_page/$', views.hpp_result_page, name='hpp_result_page'),

            url(
                r'^api/setup_request_contracts/$',
                views.api_setup_request_contracts,
                name='api_setup_request_contracts'
            ),
            url(
                r'^api/execute_request_contracts/$',
                views.api_execute_request_contracts,
                name='api_execute_request_contracts'
            ),

            url(
                r'^api/setup_disable_recurring_contract/$',
                views.api_setup_disable_recurring_contract,
                name='api_setup_disable_recurring_contract'
            ),
            url(
                r'^api/execute_disable_recurring_contract/$',
                views.api_execute_disable_recurring_contract,
                name='api_execute_disable_recurring_contract'
            ),

            url(
                r'^api/setup_recurring_session/$',
                views.api_setup_recurring_session,
                name='api_setup_recurring_session'
            ),
            url(
                r'^api/execute_recurring_session/$',
                views.api_execute_recurring_session,
                name='api_execute_recurring_session'
            ),

            url(r'^notification/$', views.parse_notification, name='parse_notification')
        ]

        return urlpatterns, 'adyengo', 'adyengo'


site = AdyengoSite()



# -*- coding: utf-8 -*-
# Generated by Django 1.9 on 2016-05-27 21:41
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('adyengo', '0006_auto_20160527_2051'),
    ]

    operations = [
        migrations.AlterField(
            model_name='notification',
            name='currency_code',
            field=models.CharField(choices=[('EUR', 'Euro')], default='EUR', max_length=3),
        ),
        migrations.AlterField(
            model_name='notification',
            name='event_code',
            field=models.CharField(blank=True, max_length=50, null=True),
        ),
        migrations.AlterField(
            model_name='notification',
            name='event_date',
            field=models.DateTimeField(blank=True, null=True),
        ),
        migrations.AlterField(
            model_name='notification',
            name='ip_address',
            field=models.CharField(blank=True, max_length=45, null=True),
        ),
        migrations.AlterField(
            model_name='notification',
            name='merchant_account_code',
            field=models.CharField(blank=True, max_length=150, null=True),
        ),
        migrations.AlterField(
            model_name='notification',
            name='merchant_reference',
            field=models.CharField(blank=True, max_length=128, null=True),
        ),
        migrations.AlterField(
            model_name='notification',
            name='operations',
            field=models.CharField(blank=True, max_length=100, null=True),
        ),
        migrations.AlterField(
            model_name='notification',
            name='original_reference',
            field=models.CharField(blank=True, max_length=150, null=True),
        ),
        migrations.AlterField(
            model_name='notification',
            name='payment_method',
            field=models.CharField(blank=True, choices=[('bankTransfer_DE', 'German Banktransfer'), ('directEbanking', 'SofortUberweisung'), ('paypal', 'PayPal'), ('amex', 'Amex'), ('bankTransfer', 'All banktransfers'), ('mc', 'Master Card'), ('bankTransfer_NL', 'Dutch Banktransfer'), ('visa', 'Visa'), ('ideal', 'iDEAL'), ('card', 'All debit and credit cards'), ('elv', 'ELV'), ('directdebit_NL', 'Direct Debit (Netherlands)')], max_length=50, null=True),
        ),
        migrations.AlterField(
            model_name='notification',
            name='psp_reference',
            field=models.CharField(blank=True, max_length=150, null=True),
        ),
        migrations.AlterField(
            model_name='notification',
            name='reason',
            field=models.CharField(blank=True, max_length=250, null=True),
        ),
        migrations.AlterField(
            model_name='notification',
            name='session',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='adyengo.Session'),
        ),
        migrations.AlterField(
            model_name='recurringpaymentresult',
            name='result_code',
            field=models.CharField(choices=[('Error', 'Error'), ('Refused', 'Refused'), ('Authorised', 'Authorised')], max_length=30),
        ),
        migrations.AlterField(
            model_name='session',
            name='country_code',
            field=models.CharField(blank=True, choices=[('BE', 'Belgium'), ('GB', 'United Kingdom'), ('DE', 'Germany'), ('NL', 'Netherlands')], max_length=2),
        ),
        migrations.AlterField(
            model_name='session',
            name='currency_code',
            field=models.CharField(choices=[('EUR', 'Euro')], default='EUR', max_length=3),
        ),
        migrations.AlterField(
            model_name='session',
            name='page_type',
            field=models.CharField(choices=[('multiple', 'Multiple'), ('skip', 'Skip'), ('single', 'Single')], default='multiple', max_length=15),
        ),
        migrations.AlterField(
            model_name='session',
            name='recurring_contract',
            field=models.CharField(blank=True, choices=[('ONECLICK', 'One click'), ('RECURRING,ONECLICK', 'Recurring and One click (user chooses)'), ('RECURRING', 'Recurring')], max_length=50),
        ),
        migrations.AlterField(
            model_name='session',
            name='session_type',
            field=models.CharField(choices=[('api_recurring', 'API Recurring'), ('hpp_regular', 'HPP Regular'), ('hpp_recurring', 'HPP Recurring')], max_length=25),
        ),
        migrations.AlterField(
            model_name='session',
            name='shopper_locale',
            field=models.CharField(blank=True, choices=[('nl_NL', 'Dutch (Holland)'), ('en_GB', 'English (United Kingdom)'), ('de_DE', 'German (Germany)'), ('nl_BE', 'Dutch (Belgium)'), ('fr_BE', 'French (Belgium)')], default='nl_NL', max_length=5),
        ),
        migrations.AlterField(
            model_name='session',
            name='skin_code',
            field=models.CharField(default='BCS1MHG2', max_length=10),
        ),
        migrations.AlterField(
            model_name='sessionallowedpaymentmethods',
            name='method',
            field=models.CharField(choices=[('bankTransfer_DE', 'German Banktransfer'), ('directEbanking', 'SofortUberweisung'), ('paypal', 'PayPal'), ('amex', 'Amex'), ('bankTransfer', 'All banktransfers'), ('mc', 'Master Card'), ('bankTransfer_NL', 'Dutch Banktransfer'), ('visa', 'Visa'), ('ideal', 'iDEAL'), ('card', 'All debit and credit cards'), ('elv', 'ELV'), ('directdebit_NL', 'Direct Debit (Netherlands)')], max_length=50),
        ),
        migrations.AlterField(
            model_name='sessionblockedpaymentmethods',
            name='method',
            field=models.CharField(choices=[('bankTransfer_DE', 'German Banktransfer'), ('directEbanking', 'SofortUberweisung'), ('paypal', 'PayPal'), ('amex', 'Amex'), ('bankTransfer', 'All banktransfers'), ('mc', 'Master Card'), ('bankTransfer_NL', 'Dutch Banktransfer'), ('visa', 'Visa'), ('ideal', 'iDEAL'), ('card', 'All debit and credit cards'), ('elv', 'ELV'), ('directdebit_NL', 'Direct Debit (Netherlands)')], max_length=50),
        ),
    ]



import uuid
import dateutil.parser

from django.shortcuts import render
from django.utils import timezone
from django.views.decorators.csrf import csrf_exempt
from django.core.urlresolvers import reverse

from .models import (
    Session, SessionAllowedPaymentMethods, SessionBlockedPaymentMethods,
    RecurringContract
)
from . import constants, settings, utils, notification


def hpp_setup_session(request, session_type):

    if not session_type:
        session_type = constants.SESSION_TYPE_HPP_REGULAR

    tomorrow = (timezone.now() + timezone.timedelta(days=1))

    params = {
        'currency_code': settings.DEFAULT_CURRENCY_CODE,
        'merchant_reference': uuid.uuid4(),
        'payment_amount': 1000,
        'session_validity': tomorrow,
        'ship_before_date': tomorrow,
        'shopper_locale': settings.DEFAULT_SHOPPER_LOCALE,
        'skin_code': settings.DEFAULT_SKIN_CODE,
        'shopper_reference': uuid.uuid4(),
        'res_url': '{}://{}{}'.format(request.scheme, request.get_host(), reverse('adyengo:hpp_result_page'))
    }

    return render(request, 'adyengo/hpp/setup_session.html', {
        'session_type_hpp_regular': constants.SESSION_TYPE_HPP_REGULAR,
        'session_type_hpp_recurring': constants.SESSION_TYPE_HPP_RECURRING,
        'session_type': session_type,
        'currency_codes': constants.CURRENCY_CODES,
        'locales': constants.LOCALES,
        'payment_methods': constants.PAYMENT_METHODS,
        'recurring_contract_types': constants.RECURRING_CONTRACT_TYPES_PLUS_COMBOS,
        'dispatch_url': reverse('adyengo:hpp_dispatch_session'),
        'params': params
    })


def hpp_dispatch_session(request):

    def p(p):
        return request.POST.get(p, '')

    session = Session(
        session_type=p('session_type'),
        merchant_reference=p('merchant_reference'),
        payment_amount=p('payment_amount'),
        currency_code=p('currency_code'),
        ship_before_date=dateutil.parser.parse(p('ship_before_date')),
        skin_code=p('skin_code'),
        session_validity=dateutil.parser.parse(p('session_validity')),
        shopper_reference=p('shopper_reference'),
        shopper_email=p('shopper_email'),
        recurring_contract=p('recurring_contract'),
        page_type=p('page_type'),
        res_url=p('res_url')
    )
    session.save()

    if request.POST.get('allowed_payment_methods'):
        for m in request.POST.getlist('allowed_payment_methods'):
            session.allowed_payment_methods.add(
                SessionAllowedPaymentMethods(method=m),
                bulk=False
            )

    if request.POST.get('blocked_payment_methods'):
        for m in request.POST.getlist('blocked_payment_methods'):
            session.blocked_payment_methods.add(
                SessionBlockedPaymentMethods(method=m),
                bulk=False
            )

    return render(request, 'adyengo/hpp/dispatch_session.html', {
        'url': session.url(),
        'params': session.hpp_params()
    })


def hpp_result_page(request):

    def p(p):
        return request.GET.get(p)

    params = {
        'authResult': p('authResult'),
        'merchantReference': p('merchantReference'),
        'merchantReturnData': p('merchantReturnData'),
        'paymentMethod': p('paymentMethod'),
        'pspReference': p('pspReference'),
        'shopperLocale': p('shopperLocale'),
        'skinCode': p('skinCode')
    }

    merchant_sig_valid = utils.merchant_sig(params) == p('merchantSig')

    return render(request, 'adyengo/hpp/result_page.html', {
        'merchant_sig_valid': merchant_sig_valid,
        'params': params
    })


def api_setup_request_contracts(request):
    return render(request, 'adyengo/api/setup_request_contracts.html', {
        'execute_url': reverse('adyengo:api_execute_request_contracts')
    })


def api_execute_request_contracts(request):

    contracts = RecurringContract.objects.contracts(
        request.POST.get('shopper_reference'),
        request.POST.get('contract_type')
    )

    return render(request, 'adyengo/api/execute_request_contracts.html', {
        'contracts': contracts
    })


def api_setup_disable_recurring_contract(request):
    return render(request, 'adyengo/api/setup_disable_recurring_contract.html', {
        'execute_url': reverse('adyengo:api_execute_disable_recurring_contract')
    })


def api_execute_disable_recurring_contract(request):

    success = RecurringContract.objects.disable(
        request.POST.get('shopper_reference'),
        request.POST.get('recurring_detail_reference')
    )

    return render(request, 'adyengo/api/execute_disable_recurring_contract.html', {
        'success': success
    })


def api_setup_recurring_session(request):

    params = {
        'merchant_reference': uuid.uuid4(),
        'selected_recurring_detail_reference': 'LATEST',
        'payment_amount': 1000,
        'currency_code': settings.DEFAULT_CURRENCY_CODE
    }

    return render(request, 'adyengo/api/setup_recurring_session.html', {
        'execute_url': reverse('adyengo:api_execute_recurring_session'),
        'params': params,
        'currency_codes': constants.CURRENCY_CODES,
        'recurring_contract_types': constants.RECURRING_CONTRACT_TYPES,
    })


def api_execute_recurring_session(request):

    def p(p):
        # Get post parameter
        return request.POST.get(p, '')

    session = Session(
        session_type=constants.SESSION_TYPE_API_RECURRING,
        merchant_reference=p('merchant_reference'),
        recurring_detail_reference=p('recurring_detail_reference'),
        recurring_contract=p('recurring_contract'),
        payment_amount=p('payment_amount'),
        currency_code=p('currency_code'),
        shopper_reference=p('shopper_reference'),
        shopper_email=p('shopper_email'),
        fraud_offset=int(p('fraud_offset')) if p('fraud_offset') else None,
        shopper_ip=p('shopper_ip'),
        shopper_statement=p('shopper_statement')
    )
    session.save()

    result = session.exec_recurring_payment()

    return render(request, 'adyengo/api/execute_recurring_session.html', {
        'result': result
    })


@csrf_exempt
def parse_notification(request):
    try:
        notification.parse_notification(request)
    except notification.InvalidNotification as error:
        return notification.notification_invalid_response(str(error))
    else:
        return notification.notification_valid_response()



from django.conf import settings
from . import constants


adyengo_settings = getattr(settings, 'ADYENGO', {})

# Your Adyen merchant account
MERCHANT_ACCOUNT = adyengo_settings.get('MERCHANT_ACCOUNT', None)

# The HMAC key used to validate the communication with Adyen
HMAC_KEY = adyengo_settings.get('HMAC_KEY', None)

# The HMAC key used to validate notifications from Adyen
NOTIFICATION_HMAC_KEY = adyengo_settings.get('NOTIFICATION_HMAC_KEY', None)

# The mode the Adyen app will work in, either 'test' or 'live'
MODE = adyengo_settings.get('MODE', 'test')

# The default skin code. This option isn't required, but if it's not set, the
# system expects that the skin code will be provided when creating the session.
DEFAULT_SKIN_CODE = adyengo_settings.get('DEFAULT_SKIN_CODE', None)

# The default page type for an Adyen HPP (Hosted Payment Pages) payment
DEFAULT_PAGE_TYPE = adyengo_settings.get('DEFAULT_PAGE_TYPE', constants.PAGE_TYPE_MULTIPLE)

# The default shopper locale. This option isn't required, but if it's not set, the
# system expects that the skin code will be provided when creating the session.
DEFAULT_SHOPPER_LOCALE = adyengo_settings.get('DEFAULT_SHOPPER_LOCALE', None)

# The default currency code. This option isn't required, but if it's not set,
# the system expects that the currency code will be provided when creating the
# session.
DEFAULT_CURRENCY_CODE = adyengo_settings.get('DEFAULT_CURRENCY_CODE', None)

# The default URL the user gets redirected to after an Adyen HPP (Hosted
# Payment Pages) payment.
DEFAULT_RES_URL = adyengo_settings.get('DEFAULT_RES_URL', None)

# The credentials for the Adyen API
API_USERNAME = adyengo_settings.get('API_USERNAME', None)
API_PASSWORD = adyengo_settings.get('API_PASSWORD', None)

if MODE == 'test':
    PAYMENT_PAGES_MULTIPLE_URL = 'https://test.adyen.com/hpp/select.shtml'
    PAYMENT_PAGES_SINGLE_URL = 'https://test.adyen.com/hpp/pay.shtml'
    PAYMENT_PAGES_SKIP_URL = 'https://test.adyen.com/hpp/details.shtml'
    PAYMENT_API_BASE_URL = 'https://pal-test.adyen.com/pal/servlet/Payment/v12/'
    RECURRING_API_BASE_URL = 'https://pal-test.adyen.com/pal/servlet/Recurring/v12/'

if MODE == 'live':
    PAYMENT_PAGES_MULTIPLE_URL = 'https://live.adyen.com/hpp/select.shtml'
    PAYMENT_PAGES_SINGLE_URL = 'https://live.adyen.com/hpp/pay.shtml'
    PAYMENT_PAGES_SKIP_URL = 'https://live.adyen.com/hpp/details.shtml'
    PAYMENT_API_BASE_URL = 'https://pal-live.adyen.com/pal/servlet/Payment/v12/'
    RECURRING_API_BASE_URL = 'https://pal-live.adyen.com/pal/servlet/Recurring/v12/'



import json
import dateutil.parser
from django.http import JsonResponse
from .models import Notification
from .utils import is_valid_ip, get_client_ip, is_notification_item_hmac_valid


class InvalidNotification(Exception):
    pass


class InvalidIp(InvalidNotification):
    pass


class HmacCalcInvalid(InvalidNotification):
    pass


def parse_notification(request):

    client_ip = get_client_ip(request)

    if not is_valid_ip(client_ip):
        raise InvalidIp('Invalid IP')

    data = json.loads(request.body.decode())

    def get_notification_items(data):
        return [
            i['NotificationRequestItem']
            for i in data.get('notificationItems', [])
        ]

    def save_notification_item(item):

        if not is_notification_item_hmac_valid(item):
            raise HmacCalcInvalid('HMAC calculation invalid')

        def adyen_bool(var):
            """
            Because Adyen likes to send strings where you expect bools..
            """
            if type(var) == bool:
                return var
            else:
                return var.lower() == 'true'

        def get_event_date():
            if item.get('eventDate'):
                return dateutil.parser.parse(item.get('eventDate'))

        amount = item.get('amount', {})

        # Adyen notificatins are likely to be sent twice, hence we use
        # `get_or_create()`.
        return Notification.objects.get_or_create(
            live=adyen_bool(data.get('live')),
            event_code=item.get('eventCode'),
            psp_reference=item.get('pspReference'),
            merchant_account_code=item.get('merchantAccountCode'),
            success=adyen_bool(item.get('success')),
            defaults={
                'ip_address': client_ip,
                'original_reference': item.get('originalReference'),
                'merchant_reference': item.get('merchantReference'),
                'event_date': get_event_date(),
                'payment_method': item.get('paymentMethod'),
                'operations': ','.join(item.get('operations', [])),
                'reason': item.get('reason'),
                'payment_amount': amount.get('value'),
                'currency_code': amount.get('currency')
            }
        )[0]  # `get_or_create()` returns `(obj, created)`, hence the `[0]`

    return [
        save_notification_item(item)
        for item in get_notification_items(
            json.loads(request.body.decode())
        )
    ]


def notification_valid_response():
    return JsonResponse({'notificationResponse': '[accepted]'})


def notification_invalid_response(error_message='Invalid Notification'):
    response = JsonResponse({'notificationResponse': error_message})
    response.status_code = 403
    return response



# -*- coding: utf-8 -*-
# Generated by Django 1.9 on 2016-10-13 16:48
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('adyengo', '0017_auto_20161013_1848'),
    ]

    operations = [
        migrations.AlterField(
            model_name='notification',
            name='payment_method',
            field=models.CharField(blank=True, choices=[('mc', 'Master Card'), ('dotpay', 'Dotpay'), ('bankTransfer_DE', 'German Banktransfer'), ('giropay', 'GiroPay'), ('diners', 'Diners Club'), ('visa', 'Visa'), ('bcmc', 'Bancontact card'), ('bankTransfer_IBAN', 'International Bank Transfer (IBAN)'), ('directdebit_NL', 'Direct Debit (Netherlands)'), ('discover', 'Discover'), ('bankTransfer', 'All banktransfers'), ('maestro', 'Maestro'), ('sepadirectdebit', 'SEPA Direct Debit'), ('amex', 'Amex'), ('directEbanking', 'SofortUberweisung'), ('bankTransfer_NL', 'Dutch Banktransfer'), ('ebanking_FI', 'Finnish E-Banking'), ('ideal', 'iDEAL'), ('card', 'All debit and credit cards'), ('elv', 'ELV'), ('paypal', 'PayPal')], max_length=50, null=True),
        ),
        migrations.AlterField(
            model_name='session',
            name='country_code',
            field=models.CharField(choices=[('NL', 'Netherlands'), ('GB', 'United Kingdom'), ('DE', 'Germany'), ('BE', 'Belgium')], max_length=2),
        ),
        migrations.AlterField(
            model_name='session',
            name='page_type',
            field=models.CharField(choices=[('skip', 'Skip'), ('single', 'Single'), ('multiple', 'Multiple')], default='skip', max_length=15),
        ),
        migrations.AlterField(
            model_name='session',
            name='recurring_contract',
            field=models.CharField(blank=True, choices=[('RECURRING,ONECLICK', 'Recurring and One click (user chooses)'), ('ONECLICK', 'One click'), ('RECURRING', 'Recurring')], max_length=50),
        ),
        migrations.AlterField(
            model_name='session',
            name='session_type',
            field=models.CharField(choices=[('api_recurring', 'API Recurring'), ('hpp_recurring', 'HPP Recurring'), ('hpp_regular', 'HPP Regular')], max_length=25),
        ),
        migrations.AlterField(
            model_name='session',
            name='shopper_locale',
            field=models.CharField(blank=True, choices=[('de_DE', 'German (Germany)'), ('fr_BE', 'French (Belgium)'), ('nl_NL', 'Dutch (Holland)'), ('nl_BE', 'Dutch (Belgium)'), ('en_GB', 'English (United Kingdom)')], default='nl_NL', max_length=5),
        ),
        migrations.AlterField(
            model_name='sessionallowedpaymentmethods',
            name='method',
            field=models.CharField(choices=[('mc', 'Master Card'), ('dotpay', 'Dotpay'), ('bankTransfer_DE', 'German Banktransfer'), ('giropay', 'GiroPay'), ('diners', 'Diners Club'), ('visa', 'Visa'), ('bcmc', 'Bancontact card'), ('bankTransfer_IBAN', 'International Bank Transfer (IBAN)'), ('directdebit_NL', 'Direct Debit (Netherlands)'), ('discover', 'Discover'), ('bankTransfer', 'All banktransfers'), ('maestro', 'Maestro'), ('sepadirectdebit', 'SEPA Direct Debit'), ('amex', 'Amex'), ('directEbanking', 'SofortUberweisung'), ('bankTransfer_NL', 'Dutch Banktransfer'), ('ebanking_FI', 'Finnish E-Banking'), ('ideal', 'iDEAL'), ('card', 'All debit and credit cards'), ('elv', 'ELV'), ('paypal', 'PayPal')], max_length=50),
        ),
        migrations.AlterField(
            model_name='sessionblockedpaymentmethods',
            name='method',
            field=models.CharField(choices=[('mc', 'Master Card'), ('dotpay', 'Dotpay'), ('bankTransfer_DE', 'German Banktransfer'), ('giropay', 'GiroPay'), ('diners', 'Diners Club'), ('visa', 'Visa'), ('bcmc', 'Bancontact card'), ('bankTransfer_IBAN', 'International Bank Transfer (IBAN)'), ('directdebit_NL', 'Direct Debit (Netherlands)'), ('discover', 'Discover'), ('bankTransfer', 'All banktransfers'), ('maestro', 'Maestro'), ('sepadirectdebit', 'SEPA Direct Debit'), ('amex', 'Amex'), ('directEbanking', 'SofortUberweisung'), ('bankTransfer_NL', 'Dutch Banktransfer'), ('ebanking_FI', 'Finnish E-Banking'), ('ideal', 'iDEAL'), ('card', 'All debit and credit cards'), ('elv', 'ELV'), ('paypal', 'PayPal')], max_length=50),
        ),
    ]
